//
//  CPDFDisplayViewController.m
//  compdfkit-tools
//
//  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
//
//  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
//  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
//  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
//  This notice may not be removed from this file.
//

#import "CPDFDisplayViewController.h"

#import <ComPDFKit/ComPDFKit.h>

#import "CPDFDisplayTableViewCell.h"
#import "CActivityIndicatorView.h"

typedef NS_ENUM(NSUInteger, CDisplayPDFType) {
    CDisplayPDFTypeSinglePage = 0,
    CDisplayPDFTypeTwoPages,
    CDisplayPDFTypeBookMode,
    CDisplayPDFTypeContinuousScroll,
    CDisplayPDFTypeCropMode,
    CDisplayPDFTypeVerticalScrolling,
    CDisplayPDFTypeThemesLight,
    CDisplayPDFTypeThemesDark,
    CDisplayPDFTypeThemesSepia,
    CDisplayPDFTypeThemesReseda
};

#pragma mark - CPDFDisplayModel

@interface CPDFDisplayModel : NSObject

@property(nonatomic, strong) UIImage *image;

@property(nonatomic, strong) NSString *titilName;

@property(nonatomic, assign) CDisplayPDFType tag;

- (instancetype)initWithType:(CDisplayPDFType)displayType;

@end

@implementation CPDFDisplayModel

- (instancetype)initWithType:(CDisplayPDFType)displayType {
    if(self = [super init]) {
        switch (displayType) {
            case CDisplayPDFTypeSinglePage:
                self.image = [UIImage imageNamed:@"CDisplayImageNameSinglePage"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                
                self.titilName = NSLocalizedString(@"Single Page", nil);
                break;
            case CDisplayPDFTypeTwoPages:
                self.image = [UIImage imageNamed:@"CDisplayImageNameTwoPages"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Two Pages", nil);
                break;
                
            case CDisplayPDFTypeBookMode:
                self.image = [UIImage imageNamed:@"CDisplayImageNameBookMode"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Book Mode", nil);

                break;
            case CDisplayPDFTypeContinuousScroll:
                self.image = [UIImage imageNamed:@"CDisplayImageNameContinuousScroll"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Continuous Scroll", nil);
                break;
            case CDisplayPDFTypeCropMode:
                self.image = [UIImage imageNamed:@"CDisplayImageNameCropMode"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Crop Mode", nil);
                break;
            case CDisplayPDFTypeVerticalScrolling:
                self.image = [UIImage imageNamed:@"CDisplayImageNameVerticalScrolling"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Vertical Scrolling", nil);
                break;
            case CDisplayPDFTypeThemesLight:
                self.image = [UIImage imageNamed:@"CDisplayImageNameThemesLight"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Light", nil);
                break;
            case CDisplayPDFTypeThemesDark:
                self.image = [UIImage imageNamed:@"CDisplayImageNameThemesDark"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Dark", nil);
                break;
            case CDisplayPDFTypeThemesSepia:
                self.image = [UIImage imageNamed:@"CDisplayImageNameThemesSepia"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Sepia", nil);
                break;
            default:
            case CDisplayPDFTypeThemesReseda:
                self.image = [UIImage imageNamed:@"CDisplayImageNameThemesReseda"
                                        inBundle:[NSBundle bundleForClass:self.class]
                   compatibleWithTraitCollection:nil];
                self.titilName = NSLocalizedString(@"Reseda", nil);
                break;
        }
        self.tag = displayType;
    }
    return self;
}

@end

#pragma mark - CPDFDisplayViewController

@interface CPDFDisplayViewController () <UITableViewDelegate,UITableViewDataSource>

@property (nonatomic, strong) UITableView* tableView;

@property (nonatomic, strong) NSArray* displayModeArray;

@property (nonatomic, strong) NSArray* themesArray;

@property (nonatomic, strong) CPDFView * pdfview;

@end

@implementation CPDFDisplayViewController

#pragma mark - Initializers

- (instancetype)initWithPDFView:(CPDFView *)pdfview {
    if(self = [super init]) {
        self.pdfview = pdfview;
        [self updateDisplayView];
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.tableView = [[UITableView alloc] initWithFrame:self.view.bounds style:UITableViewStyleGrouped];
    self.tableView.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight);
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    self.tableView.tableFooterView = [[UIView alloc] init];
    [self.view addSubview:self.tableView];
    [self.tableView reloadData];
    // Do any additional setup after loading the view.
}

#pragma mark - Accessors

- (NSArray *)displayModeArray {
    if(!_displayModeArray) {
        _displayModeArray = @[@(CDisplayPDFTypeSinglePage),
                              @(CDisplayPDFTypeTwoPages),
                              @(CDisplayPDFTypeBookMode),
                              @(CDisplayPDFTypeContinuousScroll),
                              @(CDisplayPDFTypeCropMode),
                              @(CDisplayPDFTypeVerticalScrolling)];
    }
    return _displayModeArray;
}

- (NSArray *)themesArray {
    if(!_themesArray) {
        _themesArray = @[@(CDisplayPDFTypeThemesLight),
                         @(CDisplayPDFTypeThemesDark),
                         @(CDisplayPDFTypeThemesSepia),
                         @(CDisplayPDFTypeThemesReseda)];
    }
    return _themesArray;
}

- (BOOL)isSinglePage {
    if(!self.pdfview.displayTwoUp)
        return YES;
    
    return NO;
}

- (void)setIsSinglePage:(BOOL)isSinglePage {
    self.pdfview.displayTwoUp = NO;
    self.pdfview.displaysAsBook = NO;
    [self.pdfview layoutDocumentView];
}

- (BOOL)isTwoPage {
    if(self.pdfview.displayTwoUp && !self.pdfview.displaysAsBook)
        return YES;
    
    return NO;
}

- (void)setIsTwoPage:(BOOL)isTwoPage {
    self.pdfview.displayTwoUp = YES;
    self.pdfview.displaysAsBook = NO;
    [self.pdfview layoutDocumentView];
}

- (BOOL)isBookMode {
    if(self.pdfview.displayTwoUp && self.pdfview.displaysAsBook)
        return YES;
    return NO;
}

- (void)setIsBookMode:(BOOL)isBookMode {
    self.pdfview.displayTwoUp = YES;
    self.pdfview.displaysAsBook = YES;
    [self.pdfview layoutDocumentView];
}

#pragma mark - Public method

- (void)updateDisplayView {
    [self.tableView reloadData];
}

#pragma mark - Private method

- (void)changePDFViewCropMode:(BOOL)isCropMode {
    [self.navigationController.view setUserInteractionEnabled:YES];
    CActivityIndicatorView *loadingView = [[CActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    loadingView.center = self.view.center;
    loadingView.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    if (![loadingView superview]) {
        [self.view addSubview:loadingView];
    }
    [loadingView startAnimating];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        self.pdfview.displayCrop = isCropMode;

        dispatch_async(dispatch_get_main_queue(), ^{
            [self.pdfview layoutDocumentView];
            
            [loadingView stopAnimating];
            [loadingView removeFromSuperview];
            [self.navigationController.view setUserInteractionEnabled:NO];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
        });
    });
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (0 == section)
        return self.displayModeArray.count;
     else
        return self.themesArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    CPDFDisplayTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"cell"];
    if (!cell) {
        cell = [[CPDFDisplayTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"cell"];
    }
    
    CPDFDisplayModel *model = nil;
    if(0 == indexPath.section) {
        model = [[CPDFDisplayModel alloc]initWithType:[[self.displayModeArray objectAtIndex:indexPath.row] integerValue]];
    } else {
        model = [[CPDFDisplayModel alloc]initWithType:[[self.themesArray objectAtIndex:indexPath.row] integerValue]];
    }
        
    switch (model.tag) {
        case CDisplayPDFTypeSinglePage:
            if([self isSinglePage])
                cell.checkImageView.hidden = NO;
            break;
        case CDisplayPDFTypeTwoPages:
            if([self isTwoPage])
                cell.checkImageView.hidden = NO;
            break;
        case CDisplayPDFTypeBookMode:
            if([self isBookMode])
                cell.checkImageView.hidden = NO;
            break;
        case CDisplayPDFTypeContinuousScroll:
            cell.modeSwitch.hidden = NO;
            if(self.pdfview.displaysPageBreaks)
               [cell.modeSwitch setOn:YES animated:NO];
            else
                [cell.modeSwitch setOn:NO animated:NO];
            break;
        case CDisplayPDFTypeCropMode:
            cell.modeSwitch.hidden = NO;
            if(self.pdfview.displayCrop)
               [cell.modeSwitch setOn:YES animated:NO];
            else
                [cell.modeSwitch setOn:NO animated:NO];
            break;
        case CDisplayPDFTypeVerticalScrolling:
            cell.modeSwitch.hidden = NO;
            if(CPDFDisplayDirectionVertical == self.pdfview.displayDirection)
               [cell.modeSwitch setOn:YES animated:NO];
            else
                [cell.modeSwitch setOn:NO animated:NO];

            break;
        case CDisplayPDFTypeThemesLight:
            if(CPDFDisplayModeNormal == self.pdfview.displayMode)
                cell.checkImageView.hidden = NO;
            break;
        case CDisplayPDFTypeThemesDark:
            if(CPDFDisplayModeNight == self.pdfview.displayMode)
                cell.checkImageView.hidden = NO;
            break;
        case CDisplayPDFTypeThemesSepia:
            if(CPDFDisplayModeSoft == self.pdfview.displayMode)
                cell.checkImageView.hidden = NO;
            break;
        case CDisplayPDFTypeThemesReseda:
            if(CPDFDisplayModeGreen == self.pdfview.displayMode)
                cell.checkImageView.hidden = NO;
            break;
        default:
            cell.checkImageView.hidden = YES;
            cell.modeSwitch.hidden = YES;
            break;
    }
    
    cell.imageView.image = model.image;
    cell.textLabel.text = model.titilName?:@"";
    __block typeof(self) blockSelf = self;
    __block CPDFDisplayTableViewCell * blockCell = cell;

    cell.switchBlock = ^{
        if (CDisplayPDFTypeContinuousScroll == model.tag) {
            BOOL isDisplaysPageBreaks = blockCell.modeSwitch.isOn;
            blockSelf.pdfview.displaysPageBreaks = isDisplaysPageBreaks;
            [blockSelf.pdfview layoutDocumentView];
            
            if([blockSelf.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [blockSelf.delegate displayViewControllerDismiss:blockSelf];
        } else if (CDisplayPDFTypeCropMode == model.tag) {
            BOOL isCropMode = blockCell.modeSwitch.on;
            [blockSelf changePDFViewCropMode:isCropMode];
        } else if (CDisplayPDFTypeVerticalScrolling == model.tag) {
            if(blockCell.modeSwitch.on) {
                blockSelf.pdfview.displayDirection = CPDFDisplayDirectionVertical;
            } else {
                blockSelf.pdfview.displayDirection = CPDFDisplayDirectionHorizontal;
            }
            [blockSelf.pdfview layoutDocumentView];
            
            if([blockSelf.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [blockSelf.delegate displayViewControllerDismiss:blockSelf];
        }
    };
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 60.0;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    if(0 == section)
        return NSLocalizedString(@"Display Mode",nil);
     else
        return NSLocalizedString(@"Themes",nil);
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    return 30.0;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    CDisplayPDFType type = CDisplayPDFTypeCropMode;
    if(0 == indexPath.section)
        type = (CDisplayPDFType)[[self.displayModeArray objectAtIndex:indexPath.row] integerValue];
    else
        type = (CDisplayPDFType)[[self.themesArray objectAtIndex:indexPath.row] integerValue];
        
    switch (type) {
        case CDisplayPDFTypeSinglePage:
            [self setIsSinglePage:YES];
            
            [self.tableView reloadData];
            
            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        case CDisplayPDFTypeTwoPages:
            [self setIsTwoPage:YES];
            
            [self.tableView reloadData];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        case CDisplayPDFTypeBookMode:
            [self setIsBookMode:YES];
            
            [self.tableView reloadData];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        case CDisplayPDFTypeThemesLight:
            self.pdfview.displayMode = CPDFDisplayModeNormal;
            [self.pdfview layoutDocumentView];
            
            [self.tableView reloadData];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        case CDisplayPDFTypeThemesDark:
            self.pdfview.displayMode = CPDFDisplayModeNight;
            [self.pdfview layoutDocumentView];
            
            [self.tableView reloadData];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        case CDisplayPDFTypeThemesSepia:
            self.pdfview.displayMode = CPDFDisplayModeSoft;
            [self.pdfview layoutDocumentView];
            
            [self.tableView reloadData];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        case CDisplayPDFTypeThemesReseda:
            self.pdfview.displayMode = CPDFDisplayModeGreen;
            [self.pdfview layoutDocumentView];
            
            [self.tableView reloadData];

            if([self.delegate respondsToSelector:@selector(displayViewControllerDismiss:)])
               [self.delegate displayViewControllerDismiss:self];
            break;
        default:
            break;
    }
}

@end
