//
//  CPDFSearchResultsViewController.m
//  compdfkit-tools
//
//  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
//
//  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
//  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
//  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
//  This notice may not be removed from this file.
//

#import "CPDFSearchResultsViewController.h"
#import "CPDFSearchViewCell.h"

#import <ComPDFKit/ComPDFKit.h>
#import <CoreText/CoreText.h>

#define kTextSearch_Content_Length_Max 100

@interface CPDFSearchResultsViewController () <UITableViewDelegate, UITableViewDataSource>

@property (nonatomic, strong) NSArray *resultArray;
@property (nonatomic, strong) NSString *keyword;
@property (nonatomic, strong) CPDFDocument *document;
@property (nonatomic, strong) UITableView *tableView;

@end

@implementation CPDFSearchResultsViewController

#pragma mark - Initializers

- (instancetype)initWithResultArray:(NSArray *)resultArray keyword:(NSString *) keyword document:(CPDFDocument *) document {
    if (self = [super init]) {
        _resultArray = resultArray;
        _keyword = keyword;
        _document = document;
    }
    
    return self;
}

#pragma mark - Accessors

#pragma mark - UIViewController Methods

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    UIBarButtonItem *backItem = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"CPDFSearchImagePrevious" inBundle:[NSBundle bundleForClass:self.class] compatibleWithTraitCollection:nil] style:UIBarButtonItemStylePlain target:self action:@selector(buttonItemClicked_Back:)];
    self.navigationItem.leftBarButtonItem = backItem;
    
    self.title = NSLocalizedString(@"Search Results", nil);
    
    self.tableView = [[UITableView alloc] initWithFrame:self.view.frame style:UITableViewStylePlain];
    self.tableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.rowHeight = UITableViewAutomaticDimension;
    self.tableView.estimatedRowHeight = 60;
    self.tableView.tableFooterView = [[UIView alloc] init];
    [self.tableView registerClass:[CPDFSearchViewCell class] forCellReuseIdentifier:@"cell"];
    
    [self.view addSubview:self.tableView];
}

#pragma mark - Button Actions

- (void)buttonItemClicked_Back:(id)sender {
    if([self.delegate respondsToSelector:@selector(searchResultsViewControllerDismiss:)])
        [self.delegate searchResultsViewControllerDismiss:self];
}

#pragma mark - Private Methods

- (NSMutableAttributedString *)getAttributedStringWithSelection:(CPDFSelection *)selection {
    CPDFPage * currentPage = selection.page;
    NSRange range = selection.range;
    NSUInteger startLocation = 0;
    NSUInteger maxLocation = 20;
    NSInteger keyLocation = 0;
    NSUInteger maxEndLocation = 80;
    if (range.location > maxLocation) {
        startLocation = range.location - maxLocation;
        keyLocation = maxLocation;
    } else {
        startLocation = 0;
        keyLocation = range.location;
    }
    NSUInteger endLocation = 0;
    if (range.location + maxEndLocation > currentPage.numberOfCharacters) {
        endLocation = currentPage.numberOfCharacters;
    } else {
        endLocation = range.location + maxEndLocation;
    }
    
    NSMutableAttributedString * attributed  = nil;
    if (endLocation> startLocation) {
        NSString * currentString = [currentPage stringForRange:NSMakeRange(startLocation, endLocation - startLocation)];
        
        NSRange tRange = NSMakeRange(keyLocation, self.keyword.length);
        if (tRange.location != NSNotFound) {
            attributed = [[NSMutableAttributedString alloc] initWithString:currentString];
            
            NSMutableParagraphStyle *paragraphStyle = [[NSMutableParagraphStyle alloc]init];
            paragraphStyle.firstLineHeadIndent = 10.0;
            paragraphStyle.headIndent = 10.0;
            paragraphStyle.lineBreakMode = NSLineBreakByCharWrapping;
            NSDictionary* dic = [NSDictionary dictionaryWithObjectsAndKeys:[UIFont fontWithName:@"HelveticaNeue-Medium" size:14.0],NSFontAttributeName,paragraphStyle,NSParagraphStyleAttributeName,nil];
            
            NSRange range = [[attributed string] rangeOfString:[attributed string]];
            [attributed setAttributes:dic range:range];
            
            //hightlight string
            dic = [NSDictionary dictionaryWithObjectsAndKeys:[UIColor colorWithRed:1.0 green:220.0/255.0 blue:27.0/255.0 alpha:1.0],NSBackgroundColorAttributeName,nil];
            if (attributed.length > tRange.length + tRange.location) {
                [attributed addAttributes:dic range:tRange];
            }
        }
    }
    return attributed;
}

#pragma mark - UITableViewDatasource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [self.resultArray count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    NSArray *array = [self.resultArray objectAtIndex:section];
    return [array count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    CPDFSearchViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"cell"];
    if (!cell) {
        cell =  [[CPDFSearchViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"cell"];
    }
    CPDFSelection *selection = self.resultArray[indexPath.section][indexPath.row];
    cell.contentLabel.attributedText = [self getAttributedStringWithSelection:selection];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    CPDFSelection *selection = self.resultArray[indexPath.section][indexPath.row];
    NSAttributedString *attributeText = [self getAttributedStringWithSelection:selection];
    
    CGFloat cellWidth = tableView.frame.size.width;
    UIEdgeInsets padding = UIEdgeInsetsMake(10, 10, 10, 10);
    CTFramesetterRef framesetter = CTFramesetterCreateWithAttributedString((__bridge CFAttributedStringRef)attributeText);
    CGSize suggestedSize = CTFramesetterSuggestFrameSizeWithConstraints(framesetter, CFRangeMake(0, 0), NULL, CGSizeMake(cellWidth - padding.left - padding.right, CGFLOAT_MAX), NULL);
    CGFloat cellHeight = suggestedSize.height + padding.top + padding.bottom;
    CFRelease(framesetter);
    
    return cellHeight;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    return 30;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    NSArray *array = [self.resultArray objectAtIndex:section];
    CPDFSelection *selection = array.firstObject;
    NSInteger pageIndex = [self.document indexForPage:selection.page];
    NSString *pageStr = [NSString stringWithFormat:NSLocalizedString(@"Page %ld",nil), (long)(pageIndex+1)];
    NSString *countStr = [NSString stringWithFormat:@"%@", @(array.count)];
    
    UITableViewHeaderFooterView *view = [[UITableViewHeaderFooterView alloc] initWithFrame:CGRectMake(0, 0, tableView.bounds.size.width, 30)];
    view.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    view.backgroundColor = [UIColor colorWithWhite:0.8 alpha:1.0];
    
    UILabel *label = [[UILabel alloc] init];
    label.font = [UIFont boldSystemFontOfSize:16];
    label.text = pageStr;
    [label sizeToFit];
    label.frame = CGRectMake(10, 0, label.bounds.size.width, view.bounds.size.height);
    [view.contentView addSubview:label];
    
    UILabel *sublabel = [[UILabel alloc] init];
    sublabel.font = [UIFont systemFontOfSize:14];
    sublabel.text = countStr;
    sublabel.textColor = [UIColor colorWithRed:0.0 green:122.0/255.0 blue:1.0 alpha:1.0];
    [sublabel sizeToFit];
    sublabel.frame = CGRectMake(view.bounds.size.width-sublabel.bounds.size.width-10, 0,
                                sublabel.bounds.size.width, view.bounds.size.height);
    sublabel.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
    [view.contentView addSubview:sublabel];
    
    return view;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    CPDFSelection *selection = self.resultArray[indexPath.section][indexPath.row];
    if([self.delegate respondsToSelector:@selector(searchResultsView:forSelection:indexPath:)]) {
        [self.delegate searchResultsView:self forSelection:selection indexPath:indexPath];
    }
}

@end
